/*
 * 3D Engine 
 * main.c: 
 * Based on pikuma.com 3D software renderer in C
 * Copyright (c) 2021 986-Studio. All rights reserved.
 *
 * Created by Manoël Trapier on 01/03/2021.
 */

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>

#include <SDL.h>

#include <log.h>

#include <display.h>

void setup()
{
    frameBuffer = (uint32_t *)calloc(windowWidth * windowHeight, sizeof(uint32_t));
    if (!frameBuffer)
    {
        log(TLOG_PANIC, NULL, "Memory allocation error.");
        goto exit;
    }

    frameBufferTexture = SDL_CreateTexture(renderer, SDL_PIXELFORMAT_ARGB8888, SDL_TEXTUREACCESS_STREAMING, windowWidth,
                                           windowHeight);
    if (frameBufferTexture == NULL)
    {
        log(TLOG_PANIC, NULL, "SDL Texture creation error: %s", SDL_GetError());
        goto exit;
    }

exit:
    return;
}

void processInput()
{
    SDL_Event event;
    SDL_PollEvent(&event);

    switch (event.type)
    {
    case SDL_QUIT:isRunning = false;
        break;

    case SDL_KEYDOWN:
        if (event.key.keysym.sym == SDLK_ESCAPE)
        {
            isRunning = false;
        }
        break;

    default:
        break;
    }

}

void update()
{

}

void render()
{
    SDL_SetRenderDrawColor(renderer, 255, 64, 13, 255);
    SDL_RenderClear(renderer);

    drawGrid(100, 0xFF003353);

    drawRectangle(100, 100, 50, 50, 0xFF9A2378);

    renderFrameBuffer();
    clearFrameBuffer(0xFF000000);
    
    SDL_RenderPresent(renderer);
}

int main(int argc, char *argv[])
{
    int param_i;
    bool fullScreen = false;
    MAX_DEBUG_LEVEL = TLOG_DEBUG;

    log(TLOG_ALWAYS, NULL, "Booting 3D Engine (version %s)!", VERSION);

    for (param_i = 1 ; (param_i < argc) && (argv[param_i][0] == '-') ; param_i++)
    {
        switch (argv[param_i][1])
        {
        default: exit(-1); /* Option not recognized */
        case 'f': fullScreen = true; break;
        case 'w': windowWidth = atoi(argv[++param_i]);
        case 'h': windowHeight = atoi(argv[++param_i]); break;
#ifdef DYNA_LOG_LEVEL
        case 'l': MAX_DEBUG_LEVEL = atoi(argv[++param_i]); break;
#endif
        case '-': goto no_more_params; /* Could use break, but this is more clear */
        }
    }
no_more_params:

    isRunning = initialiseWindow(fullScreen);

    setup();

    while (isRunning)
    {
        processInput();
        update();
        render();
    }

    destroy_window();

    return 0;
}